@@ -14,13 +14,25 @@
 
 import mock
 
+from keystoneclient.openstack.common.apiclient import exceptions
 from os_cloud_config import keystone
 from os_cloud_config.tests import base
 
 
 class KeystoneTest(base.TestCase):
 
-    def test_initialize(self):
+    def assert_endpoint(self, host, region='regionOne', public_endpoint=None):
+        self.client.services.create.assert_called_once_with(
+            'keystone', 'identity', description='Keystone Identity Service')
+        if public_endpoint is None:
+            public_endpoint = 'http://%s:5000/v2.0' % host
+        self.client.endpoints.create.assert_called_once_with(
+            region, self.client.services.create.return_value.id,
+            public_endpoint, 'http://%s:35357/v2.0' % host,
+            'http://192.0.0.3:5000/v2.0')
+
+    @mock.patch('subprocess.check_call')
+    def test_initialize(self, check_call_mock):
         self._patch_client()
 
         keystone.initialize(
@@ -29,19 +41,30 @@ class KeystoneTest(base.TestCase):
         self.client.roles.create.assert_has_calls(
             [mock.call('admin'), mock.call('Member')])
 
-        self.client.projects.create.assert_has_calls(
+        self.client.tenants.create.assert_has_calls(
             [mock.call('admin', None), mock.call('service', None)])
 
-        self.client.projects.find.assert_called_once_with(name='admin')
+        self.client.tenants.find.assert_called_once_with(name='admin')
+        self.client.roles.find.assert_called_once_with(name='admin')
         self.client.users.create.assert_called_once_with(
             'admin', email='admin@example.org', password='adminpasswd',
-            project=self.client.projects.find.return_value)
+            tenant_id=self.client.tenants.find.return_value.id)
 
         self.client.roles.find.assert_called_once_with(name='admin')
-        self.client.roles.grant.assert_called_once_with(
+        self.client.roles.add_user_role.assert_called_once_with(
+            self.client.users.create.return_value,
             self.client.roles.find.return_value,
-            user=self.client.users.create.return_value,
-            project=self.client.projects.find.return_value)
+            self.client.tenants.find.return_value)
+
+        self.assert_endpoint('192.0.0.3')
+
+        check_call_mock.assert_called_once_with(
+            ["ssh", "-o" "StrictHostKeyChecking=no", "-t", "-l", "root",
+             "192.0.0.3", "sudo", "keystone-manage", "pki_setup",
+             "--keystone-user",
+             "$(getent passwd | grep '^keystone' | cut -d: -f1)",
+             "--keystone-group",
+             "$(getent group | grep '^keystone' | cut -d: -f1)"])
 
     def test_initialize_for_swift(self):
         self._patch_client()
@@ -57,10 +80,10 @@ class KeystoneTest(base.TestCase):
         keystone.initialize_for_heat('192.0.0.3', 'mytoken', 'heatadminpasswd')
 
         self.client.domains.create.assert_called_once_with(
-            'heat', description='Owns users and projects created by heat')
+            'heat', description='Owns users and tenants created by heat')
         self.client.users.create.assert_called_once_with(
             'heat_domain_admin',
-            description='Manages users and projects created by heat',
+            description='Manages users and tenants created by heat',
             domain=self.client.domains.create.return_value,
             password='heatadminpasswd')
         self.client.roles.find.assert_called_once_with(name='admin')
@@ -69,12 +92,38 @@ class KeystoneTest(base.TestCase):
             user=self.client.users.create.return_value,
             domain=self.client.domains.create.return_value)
 
+    def test_create_endpoint_ssl(self):
+        self._patch_client()
+
+        keystone._create_endpoint(self.client, '192.0.0.3', 'regionOne',
+                                  'keystone.example.com')
+        public_endpoint = 'https://keystone.example.com:13000/v2.0'
+        self.assert_endpoint('192.0.0.3', public_endpoint=public_endpoint)
+
+    def test_create_endpoint_region(self):
+        self._patch_client()
+
+        keystone._create_endpoint(self.client, '192.0.0.3', 'regionTwo', None)
+        self.assert_endpoint('192.0.0.3', region='regionTwo')
+
+    @mock.patch('time.sleep')
+    def test_create_roles_retry(self, sleep):
+        self._patch_client()
+        side_effect = (exceptions.ConnectionRefused,
+                       exceptions.ServiceUnavailable, mock.DEFAULT,
+                       mock.DEFAULT)
+        self.client.roles.create.side_effect = side_effect
+        keystone._create_roles(self.client)
+        sleep.assert_has_calls([mock.call(10), mock.call(10)])
+        self.client.roles.create.assert_has_calls(
+            [mock.call('admin'), mock.call('Member')])
+
     @mock.patch('os_cloud_config.keystone.ksclient.Client')
     def test_create_admin_client(self, client):
         self.assertEqual(
             client.return_value,
             keystone._create_admin_client('192.0.0.3', 'mytoken'))
-        client.assert_called_once_with(endpoint='http://192.0.0.3:35357/v3',
+        client.assert_called_once_with(endpoint='http://192.0.0.3:35357/v2.0',
                                        token='mytoken')
 
     def _patch_client(self):
